'' =================================================================================================
''
''   File........ jm_adc124s021_test.spin
''   Purpose..... Checkout ADC on the Propeller Activity Board
''   Cogs Used... 0
''   Author...... Jon "JonnyMac" McPhalen
''                Copyright (c) 2025 Jon McPhalen
''                -- see below for terms of use
''   E-mail...... jon.mcphalen@gmail.com
''   Started.....
''   Updated..... 02 APR 2025
''
'' =================================================================================================


con

  VERSION = 1_0_0


con { timing }

  _xinfreq = 5_000_000                                          ' use 5MHz crystal
  _clkmode = xtal1 + pll16x                                     ' x16 = 80MHz

  CLK_FREQ = (_clkmode >> 6) * _xinfreq                         ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                   ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                               ' ticks in 1us


con { terminal }

  BR_TERM  = 115_200                                            ' terminal baud rate


con { fixed io pins }

  RX_PGM   = 31  { I }                                          ' serial / programming
  TX_PGM   = 30  { O }

  SDA1     = 29  { I/O }                                        ' i2c / eeprom
  SCL1     = 28  { I/O }

  LED27    = 27  { O }                                          ' LEDs on PAB and FLiP
  LED26    = 26  { O }


con { app io pins }

  ADC_CS   = 21  { O }                                          ' ADC on PAB
  ADC_SCLK = 20  { O }
  ADC_MISO = 19  { I }
  ADC_MOSI = 18  { O }


con

  #true,  ON, OFF
  #false, NO, YES


obj

' main                                                          ' * master Spin cog
  time : "jm_time_80"                                           '   timing and delays (80MHz)
  prng : "jm_prng"                                              '   pseudo-random number generation
  io   : "jm_io"                                                '   simple io
  term : "jm_fullduplexserial"                                  ' * serial IO for terminal

  adc  : "jm_adc124s021"                                        '   4-ch ADC (on PAB)

' * uses cog when loaded


dat { pre-initialized }


var { globals }


pub main | ch, level, volts

  setup
  wait_for_terminal(true, 250)

  term.fstr0(string("PAB ADC Checkout\r"))
  term.fstr0(string("----------------\r"))

  repeat
    term.fstr0(string("\002\000\002"))                          ' cursor to 0,2
    repeat ch from 0 to 3
      level := adc.read(ch)                                     ' raw channel value
      volts := 5_00 * level / $FFF                              ' convert to volts
      term.fstr3(string("%d  $%.3x  %4.2fv\r"), ch, level, volts)

    time.pause(100)

  repeat
    waitcnt(0)


pub setup

'' Setup IO and objects for application

  time.start                                                    ' setup timing & delays

  prng.seed(-cnt, -cnt ~> 2, $CAFE_BABE, cnt <- 2, cnt)         ' seed randomizer

  io.start(0, 0)                                                ' clear all pins (master cog)

  term.tstart(BR_TERM)                                          ' start serial for terminal *

  adc.start(ADC_CS, ADC_SCLK, ADC_MOSI, ADC_MISO)               ' connect to ADC


pub wait_for_terminal(clear, delay)

'' Wait for terminal to be open and key pressed

  term.rxflush
  term.rx
  if (clear)
    term.tx(term#CLS)
    time.pause(delay #> 0)


con { license }

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}